<?php

declare(strict_types=1);

namespace Drupal\book_library_api\Plugin\rest\resource;

use Drupal\Core\Entity\ContentEntityInterface; 
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\rest\Attribute\RestResource;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

#[RestResource(
    id: 'book_collection_resource',
    label: new TranslatableMarkup('Book Collection Resource'),
    uri_paths: [
        'canonical' => '/api/v1/books', // Note the plural "books"
    ],
)]
final class BookCollectionResource extends ResourceBase
{
    /**
     * The current user.
     *
     * @var \Drupal\Core\Session\AccountProxyInterface
     */
    protected $currentUser;

    /**
     * The entity type manager service.
     *
     * @var \Drupal\Core\Entity\EntityTypeManagerInterface
     */
    protected $entityTypeManager;

    public function __construct(
        array $configuration,
        $plugin_id,
        $plugin_definition,
        array $serializer_formats,
        LoggerInterface $logger,
        AccountProxyInterface $currentUser,
        EntityTypeManagerInterface $entityTypeManager
    ) {
        parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
        $this->currentUser = $currentUser;
        $this->entityTypeManager = $entityTypeManager;
    }

    public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self
    {
        return new self(
            $configuration,
            $plugin_id,
            $plugin_definition,
            $container->getParameter('serializer.formats'),
            $container->get('logger.factory')->get('rest'),
            $container->get('current_user'),
            $container->get('entity_type.manager')
        );
    }

    public function permissions()
    {
        return [
            'view_books_collection' => ['title' => 'View Books Collection'],
        ];
    }

    public function get(): ResourceResponse
    {
        // Load all books in a collection that are Available.
        $entityStorage = $this->entityTypeManager->getStorage('book');
        $query = $entityStorage->getQuery();
        $query->accessCheck(FALSE);
        $query->condition('field_book_status', 1);
        $entityIds = $query->execute();
        if (empty($entityIds)) {
            return new ResourceResponse([]);
        }
        $entities = $entityStorage->loadMultiple($entityIds);
        $bookData = [];
        foreach ($entities as $entity) {
            if (!$entity instanceof ContentEntityInterface) {
                continue;
            }
            $genres = [];
            foreach ($entity->get('field_book_genre')->referencedEntities() as $genre) {
                if ($genre->hasField('field_genre_title')) {
                    $genres[] = $genre->get('field_genre_title')->value;
                }
            }
            $bookData[] = [
                'id' => $entity->id(),
                'title' => $entity->label(),
                'author' => $entity->get('field_book_author')->value,
                'publication_year' => $entity->get('field_book_publication_year')->value,
                'description' => $entity->get('field_book_description')->value,
                'genre' => $genres,
            ];
        }
        return new ResourceResponse($bookData);
    }
}